@@ -119,6 +119,7 @@ int16_t gBadPortList[] = {
     42,    // name
     43,    // nicname
     53,    // domain
+    69,    // tftp
     77,    // priv-rjs
     79,    // finger
     87,    // ttylink
@@ -136,8 +137,10 @@ int16_t gBadPortList[] = {
     119,   // nntp
     123,   // ntp
     135,   // loc-srv / epmap
+    137,   // netbios
     139,   // netbios
     143,   // imap2
+    161,   // snmp
     179,   // bgp
     389,   // ldap
     427,   // afp (alternate)
@@ -152,6 +155,7 @@ int16_t gBadPortList[] = {
     532,   // netnews
     540,   // uucp
     548,   // afp
+    554,   // rtsp
     556,   // remotefs
     563,   // nntp+ssl
     587,   // smtp (outgoing)
@@ -159,16 +163,23 @@ int16_t gBadPortList[] = {
     636,   // ldap+ssl
     993,   // imap+ssl
     995,   // pop3+ssl
+    1719,  // h323gatestat
+    1720,  // h323hostcall
+    1723,  // pptp
     2049,  // nfs
     3659,  // apple-sasl
     4045,  // lockd
+    5060,  // sip
+    5061,  // sips
     6000,  // x11
+    6566,  // sane-port
     6665,  // irc (alternate)
     6666,  // irc (alternate)
     6667,  // irc (default)
     6668,  // irc (alternate)
     6669,  // irc (alternate)
     6697,  // irc+tls
+    10080, // amanda
     0,     // Sentinel value: This MUST be zero
 };
 
@@ -1321,7 +1332,7 @@ nsIOService::AllowPort(int32_t inPort, const char* scheme, bool* _retval) {
     return NS_OK;
   }
 
-  if (port == 0) {
+  if (port <= 0 || port >= std::numeric_limits<uint16_t>::max()) {
     *_retval = false;
     return NS_OK;
   }
@@ -1331,7 +1342,6 @@ nsIOService::AllowPort(int32_t inPort, const char* scheme, bool* _retval) {
     MutexAutoLock lock(mMutex);
     restrictedPortList.Assign(mRestrictedPortList);
   }
-
   // first check to see if the port is in our blacklist:
   int32_t badPortListCnt = restrictedPortList.Length();
   for (int i = 0; i < badPortListCnt; i++) {
