@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -13,7 +13,6 @@
 #include "nsIHttpChannelInternal.h"
 #include "nsURLHelper.h"
 #include "nsIStreamConverterService.h"
-#include "nsICacheInfoChannel.h"
 #include <algorithm>
 #include "nsContentSecurityManager.h"
 #include "nsHttp.h"
@@ -22,11 +21,13 @@
 #include "nsHttpHeaderArray.h"
 #include "mozilla/AutoRestore.h"
 
-nsPartChannel::nsPartChannel(nsIChannel *aMultipartChannel, uint32_t aPartID,
-                             nsIStreamListener *aListener)
+nsPartChannel::nsPartChannel(nsIChannel* aMultipartChannel, uint32_t aPartID,
+                             nsIStreamListener* aListener)
     : mMultipartChannel(aMultipartChannel),
       mListener(aListener),
       mStatus(NS_OK),
+      mLoadFlags(0),
+      mContentDisposition(0),
       mContentLength(UINT64_MAX),
       mIsByteRangeRequest(false),
       mByteRangeStart(0),
@@ -39,8 +40,6 @@ nsPartChannel::nsPartChannel(nsIChannel *aMultipartChannel, uint32_t aPartID,
   mMultipartChannel->GetLoadGroup(getter_AddRefs(mLoadGroup));
 }
 
-nsPartChannel::~nsPartChannel() {}
-
 void nsPartChannel::InitializeByteRange(int64_t aStart, int64_t aEnd) {
   mIsByteRangeRequest = true;
 
@@ -48,31 +47,31 @@ void nsPartChannel::InitializeByteRange(int64_t aStart, int64_t aEnd) {
   mByteRangeEnd = aEnd;
 }
 
-nsresult nsPartChannel::SendOnStartRequest(nsISupports *aContext) {
-  return mListener->OnStartRequest(this, aContext);
+nsresult nsPartChannel::SendOnStartRequest(nsISupports* aContext) {
+  return mListener->OnStartRequest(this);
 }
 
-nsresult nsPartChannel::SendOnDataAvailable(nsISupports *aContext,
-                                            nsIInputStream *aStream,
+nsresult nsPartChannel::SendOnDataAvailable(nsISupports* aContext,
+                                            nsIInputStream* aStream,
                                             uint64_t aOffset, uint32_t aLen) {
-  return mListener->OnDataAvailable(this, aContext, aStream, aOffset, aLen);
+  return mListener->OnDataAvailable(this, aStream, aOffset, aLen);
 }
 
-nsresult nsPartChannel::SendOnStopRequest(nsISupports *aContext,
+nsresult nsPartChannel::SendOnStopRequest(nsISupports* aContext,
                                           nsresult aStatus) {
   // Drop the listener
   nsCOMPtr<nsIStreamListener> listener;
   listener.swap(mListener);
-  return listener->OnStopRequest(this, aContext, aStatus);
+  return listener->OnStopRequest(this, aStatus);
 }
 
 void nsPartChannel::SetContentDisposition(
-    const nsACString &aContentDispositionHeader) {
+    const nsACString& aContentDispositionHeader) {
   mContentDispositionHeader = aContentDispositionHeader;
   nsCOMPtr<nsIURI> uri;
   GetURI(getter_AddRefs(uri));
   NS_GetFilenameFromDisposition(mContentDispositionFilename,
-                                mContentDispositionHeader, uri);
+                                mContentDispositionHeader);
   mContentDisposition =
       NS_GetContentDispositionFromHeader(mContentDispositionHeader, this);
 }
@@ -97,12 +96,12 @@ NS_INTERFACE_MAP_END
 //
 
 NS_IMETHODIMP
-nsPartChannel::GetName(nsACString &aResult) {
+nsPartChannel::GetName(nsACString& aResult) {
   return mMultipartChannel->GetName(aResult);
 }
 
 NS_IMETHODIMP
-nsPartChannel::IsPending(bool *aResult) {
+nsPartChannel::IsPending(bool* aResult) {
   // For now, consider the active lifetime of each part the same as
   // the underlying multipart channel...  This is not exactly right,
   // but it is good enough :-)
@@ -110,7 +109,7 @@ nsPartChannel::IsPending(bool *aResult) {
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetStatus(nsresult *aResult) {
+nsPartChannel::GetStatus(nsresult* aResult) {
   nsresult rv = NS_OK;
 
   if (NS_FAILED(mStatus)) {
@@ -131,6 +130,12 @@ nsPartChannel::Cancel(nsresult aStatus) {
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPartChannel::GetCanceled(bool* aCanceled) {
+  *aCanceled = NS_FAILED(mStatus);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsPartChannel::Suspend(void) {
   // Suspending an individual part must not suspend the underlying
@@ -152,50 +157,42 @@ nsPartChannel::Resume(void) {
 //
 
 NS_IMETHODIMP
-nsPartChannel::GetOriginalURI(nsIURI **aURI) {
+nsPartChannel::GetOriginalURI(nsIURI** aURI) {
   return mMultipartChannel->GetOriginalURI(aURI);
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetOriginalURI(nsIURI *aURI) {
+nsPartChannel::SetOriginalURI(nsIURI* aURI) {
   return mMultipartChannel->SetOriginalURI(aURI);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetURI(nsIURI **aURI) { return mMultipartChannel->GetURI(aURI); }
+nsPartChannel::GetURI(nsIURI** aURI) { return mMultipartChannel->GetURI(aURI); }
 
 NS_IMETHODIMP
-nsPartChannel::Open(nsIInputStream **result) {
-  // This channel cannot be opened!
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-nsPartChannel::Open2(nsIInputStream **aStream) {
+nsPartChannel::Open(nsIInputStream** aStream) {
   nsCOMPtr<nsIStreamListener> listener;
   nsresult rv =
       nsContentSecurityManager::doContentSecurityCheck(this, listener);
   NS_ENSURE_SUCCESS(rv, rv);
-  return Open(aStream);
-}
 
-NS_IMETHODIMP
-nsPartChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports *aContext) {
   // This channel cannot be opened!
   return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
-nsPartChannel::AsyncOpen2(nsIStreamListener *aListener) {
+nsPartChannel::AsyncOpen(nsIStreamListener* aListener) {
   nsCOMPtr<nsIStreamListener> listener = aListener;
   nsresult rv =
       nsContentSecurityManager::doContentSecurityCheck(this, listener);
   NS_ENSURE_SUCCESS(rv, rv);
-  return AsyncOpen(listener, nullptr);
+
+  // This channel cannot be opened!
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetLoadFlags(nsLoadFlags *aLoadFlags) {
+nsPartChannel::GetLoadFlags(nsLoadFlags* aLoadFlags) {
   *aLoadFlags = mLoadFlags;
   return NS_OK;
 }
@@ -207,12 +204,22 @@ nsPartChannel::SetLoadFlags(nsLoadFlags aLoadFlags) {
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetIsDocument(bool *aIsDocument) {
+nsPartChannel::GetTRRMode(nsIRequest::TRRMode* aTRRMode) {
+  return GetTRRModeImpl(aTRRMode);
+}
+
+NS_IMETHODIMP
+nsPartChannel::SetTRRMode(nsIRequest::TRRMode aTRRMode) {
+  return SetTRRModeImpl(aTRRMode);
+}
+
+NS_IMETHODIMP
+nsPartChannel::GetIsDocument(bool* aIsDocument) {
   return NS_GetIsDocumentChannel(this, aIsDocument);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetLoadGroup(nsILoadGroup **aLoadGroup) {
+nsPartChannel::GetLoadGroup(nsILoadGroup** aLoadGroup) {
   *aLoadGroup = mLoadGroup;
   NS_IF_ADDREF(*aLoadGroup);
 
@@ -220,74 +227,75 @@ nsPartChannel::GetLoadGroup(nsILoadGroup **aLoadGroup) {
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetLoadGroup(nsILoadGroup *aLoadGroup) {
+nsPartChannel::SetLoadGroup(nsILoadGroup* aLoadGroup) {
   mLoadGroup = aLoadGroup;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetOwner(nsISupports **aOwner) {
+nsPartChannel::GetOwner(nsISupports** aOwner) {
   return mMultipartChannel->GetOwner(aOwner);
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetOwner(nsISupports *aOwner) {
+nsPartChannel::SetOwner(nsISupports* aOwner) {
   return mMultipartChannel->SetOwner(aOwner);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetLoadInfo(nsILoadInfo **aLoadInfo) {
+nsPartChannel::GetLoadInfo(nsILoadInfo** aLoadInfo) {
   return mMultipartChannel->GetLoadInfo(aLoadInfo);
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetLoadInfo(nsILoadInfo *aLoadInfo) {
+nsPartChannel::SetLoadInfo(nsILoadInfo* aLoadInfo) {
+  MOZ_RELEASE_ASSERT(aLoadInfo, "loadinfo can't be null");
   return mMultipartChannel->SetLoadInfo(aLoadInfo);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetNotificationCallbacks(nsIInterfaceRequestor **aCallbacks) {
+nsPartChannel::GetNotificationCallbacks(nsIInterfaceRequestor** aCallbacks) {
   return mMultipartChannel->GetNotificationCallbacks(aCallbacks);
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetNotificationCallbacks(nsIInterfaceRequestor *aCallbacks) {
+nsPartChannel::SetNotificationCallbacks(nsIInterfaceRequestor* aCallbacks) {
   return mMultipartChannel->SetNotificationCallbacks(aCallbacks);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetSecurityInfo(nsISupports **aSecurityInfo) {
+nsPartChannel::GetSecurityInfo(nsISupports** aSecurityInfo) {
   return mMultipartChannel->GetSecurityInfo(aSecurityInfo);
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentType(nsACString &aContentType) {
+nsPartChannel::GetContentType(nsACString& aContentType) {
   aContentType = mContentType;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetContentType(const nsACString &aContentType) {
+nsPartChannel::SetContentType(const nsACString& aContentType) {
   bool dummy;
   net_ParseContentType(aContentType, mContentType, mContentCharset, &dummy);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentCharset(nsACString &aContentCharset) {
+nsPartChannel::GetContentCharset(nsACString& aContentCharset) {
   aContentCharset = mContentCharset;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetContentCharset(const nsACString &aContentCharset) {
+nsPartChannel::SetContentCharset(const nsACString& aContentCharset) {
   mContentCharset = aContentCharset;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentLength(int64_t *aContentLength) {
+nsPartChannel::GetContentLength(int64_t* aContentLength) {
   *aContentLength = mContentLength;
   return NS_OK;
 }
@@ -299,7 +307,7 @@ nsPartChannel::SetContentLength(int64_t aContentLength) {
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentDisposition(uint32_t *aContentDisposition) {
+nsPartChannel::GetContentDisposition(uint32_t* aContentDisposition) {
   if (mContentDispositionHeader.IsEmpty()) return NS_ERROR_NOT_AVAILABLE;
 
   *aContentDisposition = mContentDisposition;
@@ -313,7 +321,7 @@ nsPartChannel::SetContentDisposition(uint32_t aContentDisposition) {
 
 NS_IMETHODIMP
 nsPartChannel::GetContentDispositionFilename(
-    nsAString &aContentDispositionFilename) {
+    nsAString& aContentDispositionFilename) {
   if (mContentDispositionFilename.IsEmpty()) return NS_ERROR_NOT_AVAILABLE;
 
   aContentDispositionFilename = mContentDispositionFilename;
@@ -322,13 +330,13 @@ nsPartChannel::GetContentDispositionFilename(
 
 NS_IMETHODIMP
 nsPartChannel::SetContentDispositionFilename(
-    const nsAString &aContentDispositionFilename) {
+    const nsAString& aContentDispositionFilename) {
   return NS_ERROR_NOT_AVAILABLE;
 }
 
 NS_IMETHODIMP
 nsPartChannel::GetContentDispositionHeader(
-    nsACString &aContentDispositionHeader) {
+    nsACString& aContentDispositionHeader) {
   if (mContentDispositionHeader.IsEmpty()) return NS_ERROR_NOT_AVAILABLE;
 
   aContentDispositionHeader = mContentDispositionHeader;
@@ -336,13 +344,13 @@ nsPartChannel::GetContentDispositionHeader(
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetPartID(uint32_t *aPartID) {
+nsPartChannel::GetPartID(uint32_t* aPartID) {
   *aPartID = mPartID;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetIsLastPart(bool *aIsLastPart) {
+nsPartChannel::GetIsLastPart(bool* aIsLastPart) {
   *aIsLastPart = mIsLastPart;
   return NS_OK;
 }
@@ -352,27 +360,27 @@ nsPartChannel::GetIsLastPart(bool *aIsLastPart) {
 //
 
 NS_IMETHODIMP
-nsPartChannel::GetIsByteRangeRequest(bool *aIsByteRangeRequest) {
+nsPartChannel::GetIsByteRangeRequest(bool* aIsByteRangeRequest) {
   *aIsByteRangeRequest = mIsByteRangeRequest;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetStartRange(int64_t *aStartRange) {
+nsPartChannel::GetStartRange(int64_t* aStartRange) {
   *aStartRange = mByteRangeStart;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetEndRange(int64_t *aEndRange) {
+nsPartChannel::GetEndRange(int64_t* aEndRange) {
   *aEndRange = mByteRangeEnd;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetBaseChannel(nsIChannel **aReturn) {
+nsPartChannel::GetBaseChannel(nsIChannel** aReturn) {
   NS_ENSURE_ARG_POINTER(aReturn);
 
   *aReturn = mMultipartChannel;
@@ -388,18 +396,18 @@ NS_IMPL_ISUPPORTS(nsMultiMixedConv, nsIStreamConverter, nsIStreamListener,
 
 // No syncronous conversion at this time.
 NS_IMETHODIMP
-nsMultiMixedConv::Convert(nsIInputStream *aFromStream, const char *aFromType,
-                          const char *aToType, nsISupports *aCtxt,
-                          nsIInputStream **_retval) {
+nsMultiMixedConv::Convert(nsIInputStream* aFromStream, const char* aFromType,
+                          const char* aToType, nsISupports* aCtxt,
+                          nsIInputStream** _retval) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 // Stream converter service calls this to initialize the actual stream converter
 // (us).
 NS_IMETHODIMP
-nsMultiMixedConv::AsyncConvertData(const char *aFromType, const char *aToType,
-                                   nsIStreamListener *aListener,
-                                   nsISupports *aCtxt) {
+nsMultiMixedConv::AsyncConvertData(const char* aFromType, const char* aToType,
+                                   nsIStreamListener* aListener,
+                                   nsISupports* aCtxt) {
   NS_ASSERTION(aListener && aFromType && aToType,
                "null pointer passed into multi mixed converter");
 
@@ -414,15 +422,20 @@ nsMultiMixedConv::AsyncConvertData(const char *aFromType, const char *aToType,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMultiMixedConv::GetConvertedType(const nsACString& aFromType,
+                                   nsIChannel* aChannel, nsACString& aToType) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 // nsIRequestObserver implementation
 NS_IMETHODIMP
-nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
+nsMultiMixedConv::OnStartRequest(nsIRequest* request) {
   // we're assuming the content-type is available at this stage
   NS_ASSERTION(mBoundary.IsEmpty(), "a second on start???");
 
   nsresult rv;
 
-  mContext = ctxt;
   mTotalSent = 0;
   mChannel = do_QueryInterface(request, &rv);
   if (NS_FAILED(rv)) return rv;
@@ -503,9 +516,8 @@ nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
 
 // nsIStreamListener implementation
 NS_IMETHODIMP
-nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
-                                  nsIInputStream *inStr, uint64_t sourceOffset,
-                                  uint32_t count) {
+nsMultiMixedConv::OnDataAvailable(nsIRequest* request, nsIInputStream* inStr,
+                                  uint64_t sourceOffset, uint32_t count) {
   // Failing these assertions may indicate that some of the target listeners of
   // this converter is looping the thead queue, which is harmful to how we
   // collect the raw (content) data.
@@ -532,8 +544,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
 }
 
 NS_IMETHODIMP
-nsMultiMixedConv::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
-                                nsresult aStatus) {
+nsMultiMixedConv::OnStopRequest(nsIRequest* request, nsresult aStatus) {
   nsresult rv;
 
   if (mBoundary.IsEmpty()) {  // no token, no love.
@@ -561,14 +572,20 @@ nsMultiMixedConv::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
     // the middle of sending data. if we were, mPartChannel,
     // above, would have been non-null.
 
-    (void)mFinalListener->OnStartRequest(request, ctxt);
-    (void)mFinalListener->OnStopRequest(request, ctxt, aStatus);
+    (void)mFinalListener->OnStartRequest(request);
+    (void)mFinalListener->OnStopRequest(request, aStatus);
+  }
+
+  nsCOMPtr<nsIMultiPartChannelListener> multiListener =
+      do_QueryInterface(mFinalListener);
+  if (multiListener) {
+    multiListener->OnAfterLastPart(aStatus);
   }
 
   return NS_OK;
 }
 
-nsresult nsMultiMixedConv::ConsumeToken(Token const &token) {
+nsresult nsMultiMixedConv::ConsumeToken(Token const& token) {
   nsresult rv;
 
   switch (mParserState) {
@@ -657,7 +674,7 @@ nsresult nsMultiMixedConv::ConsumeToken(Token const &token) {
         return rv;
       }
       mParserState = BODY;
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
     case BODY: {
       if (!token.Equals(mLFToken) && !token.Equals(mCRLFToken)) {
@@ -762,6 +779,7 @@ void nsMultiMixedConv::SwitchToControlParsing() {
 nsMultiMixedConv::nsMultiMixedConv()
     : mCurrentPartID(0),
       mInOnDataAvailable(false),
+      mResponseHeader(HEADER_UNKNOWN),
       // XXX: This is a hack to bypass the raw pointer to refcounted object in
       // lambda analysis. It should be removed and replaced when the
       // IncrementalTokenizer API is improved to avoid the need for such
@@ -781,8 +799,6 @@ nsMultiMixedConv::nsMultiMixedConv()
   mRequestListenerNotified = false;
 }
 
-nsMultiMixedConv::~nsMultiMixedConv() {}
-
 nsresult nsMultiMixedConv::SendStart() {
   nsresult rv = NS_OK;
 
@@ -805,7 +821,7 @@ nsresult nsMultiMixedConv::SendStart() {
   // before starting up another "part." that would be bad.
   MOZ_ASSERT(!mPartChannel, "tisk tisk, shouldn't be overwriting a channel");
 
-  nsPartChannel *newChannel;
+  nsPartChannel* newChannel;
   newChannel = new nsPartChannel(mChannel, mCurrentPartID++, partListener);
   if (!newChannel) return NS_ERROR_OUT_OF_MEMORY;
 
@@ -877,7 +893,7 @@ nsresult nsMultiMixedConv::SendStop(nsresult aStatus) {
   return rv;
 }
 
-void nsMultiMixedConv::AccumulateData(Token const &aToken) {
+void nsMultiMixedConv::AccumulateData(Token const& aToken) {
   if (!mRawData) {
     // This is the first read of raw data during this FeedInput loop
     // of the incremental tokenizer.  All 'raw' tokens are coming from
@@ -929,10 +945,7 @@ nsresult nsMultiMixedConv::SendData() {
   mRawData = nullptr;
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIInputStream> inStream(do_QueryInterface(ss, &rv));
-  if (NS_FAILED(rv)) return rv;
-
-  return mPartChannel->SendOnDataAvailable(mContext, inStream, offset,
+  return mPartChannel->SendOnDataAvailable(mContext, ss, offset,
                                            mRawDataLength);
 }
 
@@ -967,8 +980,7 @@ nsresult nsMultiMixedConv::ProcessHeader() {
           do_QueryInterface(mChannel);
       mResponseHeaderValue.CompressWhitespace();
       if (httpInternal) {
-        DebugOnly<nsresult> rv =
-            httpInternal->SetCookie(mResponseHeaderValue.get());
+        DebugOnly<nsresult> rv = httpInternal->SetCookie(mResponseHeaderValue);
         MOZ_ASSERT(NS_SUCCEEDED(rv));
       }
       break;
@@ -1023,12 +1035,10 @@ nsresult nsMultiMixedConv::ProcessHeader() {
   return NS_OK;
 }
 
-nsresult NS_NewMultiMixedConv(nsMultiMixedConv **aMultiMixedConv) {
-  NS_PRECONDITION(aMultiMixedConv != nullptr, "null ptr");
-  if (!aMultiMixedConv) return NS_ERROR_NULL_POINTER;
+nsresult NS_NewMultiMixedConv(nsMultiMixedConv** aMultiMixedConv) {
+  MOZ_ASSERT(aMultiMixedConv != nullptr, "null ptr");
 
-  *aMultiMixedConv = new nsMultiMixedConv();
-
-  NS_ADDREF(*aMultiMixedConv);
+  RefPtr<nsMultiMixedConv> conv = new nsMultiMixedConv();
+  conv.forget(aMultiMixedConv);
   return NS_OK;
 }
