@@ -13,9 +13,7 @@
 NS_IMPL_ISUPPORTS(nsRandomGenerator, nsIRandomGenerator)
 
 NS_IMETHODIMP
-nsRandomGenerator::GenerateRandomBytes(uint32_t aLength,
-                                       uint8_t** aBuffer)
-{
+nsRandomGenerator::GenerateRandomBytes(uint32_t aLength, uint8_t** aBuffer) {
   NS_ENSURE_ARG_POINTER(aBuffer);
   *aBuffer = nullptr;
 
@@ -25,9 +23,6 @@ nsRandomGenerator::GenerateRandomBytes(uint32_t aLength,
   }
 
   auto buf = static_cast<uint8_t*>(moz_xmalloc(aLength));
-  if (!buf) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
 
   SECStatus srv = PK11_GenerateRandomOnSlot(slot.get(), buf, aLength);
   if (srv != SECSuccess) {
